/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Debug printing facilities 
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/defs.hpp>

#include <hw/assertion.hpp>

#include <stdio.h>

#include <iostream>
#include <sstream>
#include <string>

///////////////////////////////////////////////////////////////////////////////
// Debug
///////////////////////////////////////////////////////////////////////////////

#define __CPPTESK_DEBUG_BUFFER_SIZE 1024

#define CPPTESK_DEBUG_PRINT(debug_level, debug_message) \
    do { \
        std::stringstream buffer; \
        buffer << debug_message; \
        Debug::print(debug_level, buffer.str()); \
    } while(false)

#define CPPTESK_DEBUG_PRINTF(debug_level, debug_message, ...) \
    do { \
        char buffer[__CPPTESK_DEBUG_BUFFER_SIZE]; \
        if(snprintf(buffer, __CPPTESK_DEBUG_BUFFER_SIZE, debug_message, __VA_ARGS__) >= __CPPTESK_DEBUG_BUFFER_SIZE) { \
            CPPTESK_ASSERTION(false, "CPPTESK_DEBUG_PRINTF: too long message"); \
        } else { \
            Debug::print(debug_level, buffer); \
        } \
    } while(false)

#ifdef HW_SHORT_NAMES
    #define DEBUG_PRINT(debug_level, debug_message) CPPTESK_DEBUG_PRINT(debug_level, debug_message)
#endif // HW_SHORT_NAMES

namespace cpptesk {
namespace hw {

enum debug_level_t
{
    /// Print all events.
    DEBUG_INFO    =  3,
    DEBUG_ALL     =  3,
    /// Print user messages, warnings and errors.
    DEBUG_USER    =  2,
    DEBUG_MESSAGE =  2,
    /// Debug warning and errors only.
    DEBUG_WARN    =  1,
    DEBUG_WARNING =  1,
    /// Debug errors only.
    DEBUG_FAIL    =  0,
    DEBUG_ERROR   =  0,
    /// Turn off debug printing.
    DEBUG_NONE    = -1,
};

class Debug
{
public:
    static void level(debug_level_t debug_level);
    static void print(debug_level_t debug_level, const std::string &debug_message);

private:
    static const char *_STRING[DEBUG_ALL + 1];
    static const char *_STYLE [DEBUG_ALL + 1];
    static const char *_NORMAL_STYLE;
private:
    Debug() {}
   ~Debug() {}

private:
    static debug_level_t _debug_level;
};

inline void debug(debug_level_t debug_level = DEBUG_ALL)
    { Debug::level(debug_level); }

}} // namespace cpptesk::hw

