/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdlib.h>
#include <string.h>

int parse_parameters(char *str, const char *scen_name, char ***parsed_arguments)
{
    int parsed_argc = 0;
    char **parsed_argv;

    parsed_argv = (char**)malloc((parsed_argc + 1) * sizeof(void*));
    parsed_argv[parsed_argc] = (char*)malloc((strlen(scen_name) + 1) * sizeof(char));
    strcpy(parsed_argv[parsed_argc], scen_name);
    parsed_argv[parsed_argc][strlen(scen_name)] = 0;
    parsed_argc++;

    if(str != NULL)
    {
        int i = 0;
        int start_pos = 0;
        int end_pos = 0;

        while(str[i] != 0)
        {
            int commented = 0;
            int commented_quotes = 0;

            // searching for the start of parameter
            while(str[i] == 0x20)
            {
                i++;
            }

            start_pos = i;

            // searching for the end of parameter
            while((str[i] != 0x20 || str[i] == 0x20 && commented == 1) && str[i] != 0)
            {
                if(commented_quotes == 0)
                {
                    if(commented == 1)
                    {
                        if(str[i] == '\\')
                            commented_quotes = 1;
                    }

                    if(str[i] == '\'')
                        commented = commented == 0 ? 1 : 0;
                }
                else
                {
                    commented_quotes = 0;
                }

                i++;
            }

            end_pos = i;

            // if we find something, it can be saved into argv with argc incrementation
            if(end_pos - start_pos > 0)
            {
                int j = 0;
                int k = 0;
                int size = 0;

                parsed_argv = (char**)realloc(parsed_argv, (parsed_argc + 1) * sizeof(void*));

                // computing size of symbols to copy (except screened letters)
                for(j = start_pos; j < end_pos; j++)
                {
                    if(commented_quotes == 1)
                    {
                        size++;
                        commented_quotes = 0;
                        continue;
                    }

                    if(str[j] == '\\')
                    {
                        commented_quotes = 1;
                        continue;
                    }

                    if(str[j] == '\"' || str[j] == '\'')
                        continue;

                    size++;
                }
                size++;

                parsed_argv[parsed_argc] = (char*)malloc(size * sizeof(char));

                // copying from str to parsed_argv
                for(j = start_pos; j < end_pos; j++)
                {
                    if(commented_quotes == 1)
                    {
                        parsed_argv[parsed_argc][k] = str[j];
                        k++;
                        commented_quotes = 0;
                        continue;
                    }

                    if(str[j] == '\\')
                    {
                        commented_quotes = 1;
                        continue;
                    }

                    if(str[j] == '\"' || str[j] == '\'')
                        continue;

                    parsed_argv[parsed_argc][k] = str[j];
                    k++;
                }

                parsed_argv[parsed_argc][size - 1] = 0;

                parsed_argc++;
            }
        }
    }

    *parsed_arguments = parsed_argv;
    return parsed_argc;
}

